/* -*- Mode: C; c-set-style: gnu -*- */

/*
  Copyright (C) 2001 Faried Nawaz

  Author: Faried Nawaz <fn@hungry.org>
  Maintainer: fn@hungry.org
  Status: Needs a recent GTK+ library.
  Keywords: 
  Created: 2001-07-01
  
  $Id: selections.c,v 1.6 2001/07/28 22:23:29 fn Exp $
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, you can either send email to this
  program's maintainer or write to: The Free Software Foundation,
  Inc.; 59 Temple Place, Suite 330; Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>

#define PROGNAME        	"s e l e c t i o n s"
#define PROGVERSION     	"1.1 (2001-07-29)"
#define PROGAUTHOR      	"fn; http://web.nilpotent.org/"

#define DEFAULT_STATUS_VALUE    "Ready and Waiting!"
#define GRABBING_STATUS_VALUE   "Grabbing PRIMARY..."

#define RC_FILENAME		"~/.gtk/selectionrc"
#define DEFAULT_SAVE_FILE	"selections.txt"

GtkWidget *main_window;
GtkWidget *clist;
GtkWidget *status_entry;

guchar *curr_selection;
gboolean have_selection;
gint selection_timeout_id;


void show_about_box(GtkWidget *w, gpointer data)
{
  GtkWidget *about_box;
  GtkWidget *label_progname;
  GtkWidget *label_version;
  GtkWidget *label_author;
  GtkWidget *button;

  about_box = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(about_box), "About...");
  gtk_window_set_modal(GTK_WINDOW(about_box), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(about_box), GTK_WINDOW(main_window));
  gtk_window_set_default_size(GTK_WINDOW(about_box), 162, 100);
  /* for some window managers */
  gtk_window_set_position(GTK_WINDOW(about_box), GTK_WIN_POS_MOUSE);

  label_progname = gtk_label_new(PROGNAME);
  label_version = gtk_label_new(PROGVERSION);
  label_author = gtk_label_new(PROGAUTHOR);
  button = gtk_button_new_with_label("Okay");

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_box)->vbox),
		    label_progname);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_box)->vbox),
		    label_version);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_box)->vbox),
		    label_author);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_box)->action_area),
		    button);
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(about_box));
  gtk_widget_show_all(about_box);
}

int set_status(gpointer data)
{
  guchar *text = (guchar *)data;

  gtk_entry_set_text(GTK_ENTRY(status_entry), text);
  return(FALSE);
}

void send_selection(GtkWidget *w, GtkSelectionData *sel_data,
                    guint info, guint time)
{
  gtk_selection_data_set(sel_data, GDK_SELECTION_TYPE_STRING,
                         8, curr_selection, strlen(curr_selection));
}

int selection_clear(GtkWidget *w, GdkEventSelection *ev)
{
  have_selection = FALSE;
  gtk_clist_unselect_all(GTK_CLIST(clist));

  return(TRUE);
}

int get_selection(gpointer data)
{
  gtk_selection_convert(GTK_WIDGET(data),
			GDK_SELECTION_PRIMARY,
			GDK_TARGET_STRING,
                        GDK_CURRENT_TIME);
  
  return(TRUE);
}


void start_button_clicked(GtkWidget *w, gpointer data)
{
  if (selection_timeout_id)
    return;

  selection_timeout_id =  gtk_timeout_add(750, get_selection, w);
  set_status(GRABBING_STATUS_VALUE);
}

void stop_button_clicked(GtkWidget *w, gpointer data)
{
  if (selection_timeout_id)
    gtk_timeout_remove(selection_timeout_id);

  selection_timeout_id = 0;
  set_status(DEFAULT_STATUS_VALUE);
}

int already_in_list(GtkWidget *c, gint last_row, guchar *sel_text)
{
  int x;
  int ret = 0;
  guchar *text;

  for (x = 0; x <= last_row; x++) {
    gtk_clist_get_text(GTK_CLIST(c), x, 0, (gchar **)&text);
    if (strcmp(text, sel_text) == 0) {
      ret = 1;
      break;
    }
  }
	
  return (ret);
}


void selection_received (GtkWidget *w,
                         GtkSelectionData *sel_data,
                         gpointer data)
{
  guchar *sel_text;
  static gint last_row = 0;	/* last row added */

  if (sel_data->length == -1)
    return;

  if (sel_data->type != GDK_SELECTION_TYPE_STRING)
    return;

  sel_text = (guchar *)sel_data->data;

  /* if we already have it, don't add it */
  if (already_in_list(clist, last_row, sel_text))
    return;

  if (curr_selection)
    g_free(curr_selection);
  curr_selection = g_strdup(sel_text);

  gtk_clist_freeze(GTK_CLIST(clist));
  last_row = gtk_clist_append(GTK_CLIST(clist), (gchar **)&sel_text);
  gtk_clist_thaw(GTK_CLIST(clist));

  /* is it visible?  if not, scroll. */
  if (gtk_clist_row_is_visible(GTK_CLIST(clist), last_row) != GTK_VISIBILITY_FULL)
    gtk_clist_moveto(GTK_CLIST(clist), last_row, 0, 0.5, 0.0);
}

void row_selected(GtkWidget *w, gint row, gint col,
                  GdkEventButton *ev, gpointer data)
{
  guchar *text;

  gtk_clist_get_text(GTK_CLIST(w), row, col, (gchar **)&text);

  if (curr_selection)
    g_free(curr_selection);
  curr_selection = g_strdup(text);

  /* steal it */
  have_selection = gtk_selection_owner_set(main_window,
                                           GDK_SELECTION_PRIMARY,
                                           GDK_CURRENT_TIME);

}

void really_save(GtkFileSelection *selector, gpointer user_data)
{
  GtkWidget *dialog;
  GtkWidget *label;
  GtkWidget *label2;
  GtkWidget *button;
  FILE *out;
  gchar *filename;

  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data));

  dialog = gtk_dialog_new();
  label2 = gtk_label_new(filename);
  button = gtk_button_new_with_label("Okay");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(dialog));
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), button);
  /* for some window managers */
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);

  if ((out = fopen(filename, "w")) == NULL) {
    label = gtk_label_new("Can't save to");
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label2);
  } else {
    int x;
    guchar *text;

    for (x = 0; x < GTK_CLIST(clist)->rows; x++) {
      gtk_clist_get_text(GTK_CLIST(clist), x, 0, (gchar **)&text);
      fprintf(out, text);
      if (text[strlen(text) - 1] != '\n')
	fputc('\n', out);
    }

    label = gtk_label_new("Saved to");
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label2);
  }
    gtk_widget_show_all(dialog);
}

void save_selections(GtkWidget *w, gpointer data)
{
  GtkWidget *fsb;
  
  fsb = gtk_file_selection_new("Save...");

  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fsb), DEFAULT_SAVE_FILE);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fsb)->ok_button),
		     "clicked", GTK_SIGNAL_FUNC(really_save), (gpointer)fsb);
  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fsb)->ok_button),
			    "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    (gpointer)fsb);
  
  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fsb)->cancel_button),
			    "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    (gpointer)fsb);
     
  /* for some window managers */
  gtk_window_set_position(GTK_WINDOW(fsb), GTK_WIN_POS_MOUSE);

  gtk_widget_show (fsb);
}

static GtkItemFactoryEntry menu_items[] = {
  { "/_File", NULL, NULL, 0, "<Branch>", },
  { "/File/_Save", "<control>S", save_selections, 0, NULL },
  { "/File/_Quit", "<control>Q", gtk_main_quit, 0, NULL },
  { "/_Help", NULL, NULL, 0, "<LastBranch>" },
  { "/Help/_About", "<control>A", show_about_box, 0, NULL }
};

void make_menu(GtkWidget *window, GtkWidget **menubar)
{
  GtkItemFactory *item_factory; /* menu item factory */
  GtkAccelGroup *accel_group;   /* for menu accelerators */
  gint num_menu_items = sizeof(menu_items) / sizeof(menu_items[0]);

  accel_group = gtk_accel_group_new();
  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
                                      accel_group);
  
  gtk_item_factory_create_items(item_factory, num_menu_items,
				menu_items, NULL);
  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
  
  if (menubar)
    *menubar = gtk_item_factory_get_widget(item_factory, "<main>");
}

int main (int argc, char *argv[])
{
  GtkWidget *vbox;              /* container */
  GtkWidget *menubar;           /* menus */
  GtkWidget *scrolled_window;   /* for the clist */
  GtkWidget *scrollbar;         /* vertical scrollbar */
  GtkWidget *hbox;              /* container for button + label */
  GtkWidget *status_label;      /* a label */
  GtkWidget *start_button;      /* button to start selecting */
  GtkWidget *stop_button;       /* button to stop selecting */
                     
  gtk_init(&argc, &argv);

  have_selection = FALSE;
  curr_selection = NULL;
  selection_timeout_id = 0;

  gtk_rc_parse(RC_FILENAME);

  main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(main_window), "selections");
  gtk_signal_connect(GTK_OBJECT(main_window), "destroy",
                     GTK_SIGNAL_FUNC(gtk_main_quit), "WM destroy");
  gtk_window_set_default_size(GTK_WINDOW(main_window), 450, 300);

  /* selection foo */
  gtk_selection_add_target(GTK_WIDGET(main_window),
                           GDK_SELECTION_PRIMARY,
                           GDK_TARGET_STRING, 1);
  gtk_signal_connect(GTK_OBJECT(main_window), "selection_get",
                     GTK_SIGNAL_FUNC(send_selection), NULL);
  gtk_signal_connect(GTK_OBJECT(main_window), "selection_clear_event",
                     GTK_SIGNAL_FUNC(selection_clear), NULL);

  vbox  = gtk_vbox_new(FALSE, 1);
  gtk_container_add(GTK_CONTAINER(main_window), vbox);
  gtk_widget_show(vbox);

  make_menu(main_window, &menubar);
  gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);

  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_ALWAYS);
  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show(scrolled_window);

  clist = gtk_clist_new(1);
  gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
  gtk_clist_set_shadow_type(GTK_CLIST(clist), GTK_SHADOW_OUT);
  gtk_clist_set_column_justification(GTK_CLIST(clist), 0, GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_width(GTK_CLIST(clist), 0, 400);
  gtk_signal_connect(GTK_OBJECT(clist), "select_row",
                     GTK_SIGNAL_FUNC(row_selected), NULL);
  gtk_container_add(GTK_CONTAINER(scrolled_window), clist);
  gtk_widget_show(clist);

  hbox = gtk_hbox_new(FALSE, 4);
  status_label = gtk_label_new("Status:");
  gtk_box_pack_start(GTK_BOX(hbox), status_label, FALSE, TRUE, 2);
  gtk_widget_show(status_label);

  status_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(status_entry), DEFAULT_STATUS_VALUE);
  gtk_entry_set_editable(GTK_ENTRY(status_entry), FALSE);
  gtk_box_pack_start(GTK_BOX(hbox), status_entry, FALSE, TRUE, 2);
  gtk_widget_show(status_entry);

  stop_button = gtk_button_new_with_label("Stop");
  gtk_signal_connect(GTK_OBJECT(stop_button), "clicked",
                     GTK_SIGNAL_FUNC(stop_button_clicked), NULL);
  gtk_box_pack_end(GTK_BOX(hbox), stop_button, TRUE, TRUE, 2);
  gtk_widget_show(stop_button);

  start_button = gtk_button_new_with_label("Start");
  gtk_signal_connect(GTK_OBJECT(start_button), "clicked",
                     GTK_SIGNAL_FUNC(start_button_clicked), NULL);
  gtk_signal_connect(GTK_OBJECT(start_button), "selection_received",
                     GTK_SIGNAL_FUNC(selection_received), NULL);
  gtk_box_pack_end(GTK_BOX(hbox), start_button, TRUE, TRUE, 2);
  gtk_widget_show(start_button);


  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 2);
  gtk_widget_show(hbox);

  gtk_widget_show(main_window);
                     
  gtk_main ();
                     
  return 0;
}
